import * as lambda from 'aws-cdk-lib/aws-lambda';
import { Construct } from 'constructs';
import { BaseBuildProps } from './build';
/**
 * Properties for a RustFunction
 */
export interface RustFunctionProps extends lambda.FunctionOptions, BaseBuildProps {
    /**
     * Path to directory with Cargo.toml
     *
     * @default - Directory from where cdk binary is invoked
     */
    readonly directory?: string;
    /**
     * The build directory
     *
     * @default - `.build` in the entry file directory
     */
    readonly buildDir?: string;
    /**
     * The cache directory
     *
     * Parcel uses a filesystem cache for fast rebuilds.
     *
     * @default - `.cache` in the root directory
     */
    readonly cacheDir?: string;
}
/**
 * Base layout of a `Cargo.toml` file in a Rust project
 *
 * Note: This is only used when `RustFunctionProps.bin` is not defined
 * from above.
 */
export interface CargoTomlProps {
    readonly package: {
        name: string;
    };
}
/**
 * A Rust Lambda function built using cross
 */
export declare class RustFunction extends lambda.Function {
    constructor(scope: Construct, id: string, props: RustFunctionProps);
}
