"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RustFunction = void 0;
const lambda = __importStar(require("aws-cdk-lib/aws-lambda"));
const toml = __importStar(require("toml"));
const crypto = __importStar(require("crypto"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const perf_hooks_1 = require("perf_hooks");
const build_1 = require("./build");
const DEFAULT_TARGET = 'x86_64-unknown-linux-musl';
/**
 * A Rust Lambda function built using cross
 */
class RustFunction extends lambda.Function {
    constructor(scope, id, props) {
        const entry = props.directory || process.cwd();
        const handler = 'does.not.matter';
        // Custom Runtime, running on `Amazon Linux 2`
        const runtime = lambda.Runtime.PROVIDED_AL2;
        const target = props.target || DEFAULT_TARGET;
        const buildDir = props.buildDir || path.join(entry, '.build');
        let executable;
        let binName;
        if (props.package) {
            binName = undefined;
            executable = props.package;
        }
        else {
            binName = props.bin || getPackageName(entry);
            executable = binName;
        }
        const handlerDir = path.join(buildDir, 
        // We need to generate a *unique* hash in case there are multiple
        // executables, so use the `binName` here instead.
        crypto
            .createHash('sha256')
            .update(executable)
            .digest('hex'));
        createDirectory(buildDir);
        createDirectory(handlerDir);
        let start = perf_hooks_1.performance.now();
        // Build with Parcel
        build_1.build({
            entry,
            bin: binName,
            package: props.package,
            target: target,
            outDir: handlerDir,
        });
        logTime(start, `🎯  Cross-compile \`${executable}\``);
        super(scope, id, {
            ...props,
            runtime,
            code: lambda.Code.fromAsset(handlerDir),
            handler: handler,
        });
    }
}
exports.RustFunction = RustFunction;
function createDirectory(dir) {
    if (!fs.existsSync(dir)) {
        fs.mkdirSync(dir);
    }
}
function logTime(start, message) {
    const elapsedSec = ((perf_hooks_1.performance.now() - start) / 1000).toFixed(2);
    console.log(`${message}: ${elapsedSec}s`);
}
function getPackageName(entry) {
    const tomlFilePath = path.join(entry, 'Cargo.toml');
    // console.trace(`Parsing TOML file at ${tomlFilePath}`);
    try {
        const contents = fs.readFileSync(tomlFilePath, 'utf8');
        let data = toml.parse(contents);
        return data.package.name;
    }
    catch (err) {
        throw new Error(`Unable to parse package name from \`${tomlFilePath}\`\n` +
            `  ${err}\n` +
            `  Resolution: Pass the executable as the \`bin\` parameter, ` +
            `or as \`package\` for a workspace.`);
    }
}
